#!/bin/bash


ny_array=(50 60 70  80  90  100 110 120 130 140 150)
nx_array=(80 96 112 128 144 160 176 192 208 224 240)
volume_corr=("0.4" "0.5" "0.6" "0.6" "0.7" "0.7" "0.8" "0.8" "0.9" "0.9" "1.0")
ref_dir="./V200_ref"
petsc_dir="/home/gl5972/HOME/docs/2021-RA/SecondSubmission/petsc"
python_dir="/home/gl5972/HOME/docs/2021-RA/SecondSubmission/figures"
sim="false"
ctr="false"
np="1"
times="500:701:1"


function print_usage()
{
	echo "Convergence study"
	echo "--help:       help"
	echo "--verbose:    verbose"
	echo "--ref_dir:    directory where a reference data are stored (ex. V200)"
	echo "--petsc_dir:  directory where petsc code is stored"
	echo "--python_dir: directory where python code is stored"
	echo "--sim:        petsc simulations"
	echo "--ctr:        extract bubble contours"
	echo "--np:         split each (ctr) job into np processors"
	echo "--times:      from:to:every"
	echo " bash ./ConvergenceStudy --ctr --sim --np 3 --times 500:700"
	exit
}


function pass_args()
{
	local shorts='hvr:p:y:scn:t:'
	local longs='help,verbose,ref_dir:,petsc_dir:,python_dir:,sim,ctr,np:,times:'
	local options=$(getopt --options ${shorts} --long ${longs} --name "$0" -- "$@")
	if [ $? != 0 ]; then echo "Failed parsing options." >&2 ; exit 1 ; fi
	eval set -- "${options}"
	while true; do
		case "$1" in
			-h | --help)       print_usage;    shift  ;;
			-v | --verbose)    verbose="true"; shift  ;;
			-r | --ref_dir)    ref_dir=$2;     shift 2;;
			-p | --petsc_dir)  petsc_dir=$2;   shift 2;;
			-y | --python_dir) python_dir=$2;  shift 2;;
			-s | --sim)        sim="true";     shift  ;;
			-c | --ctr)        ctr="true";     shift  ;;
			-n | --np)         np=$2;          shift 2;;
			-t | --times)      times=$2;       shift 2;;
			--) shift ; break ;;
			* ) break ;;
		esac
	done
}


function set_nx_in_input_file(){
	local nx=${1}
	local input_file=${2}
	if [ -f ${input_file} ]; then
		local line_number=$(nl -b a ${input_file} | grep "[^a-z]nx .*=" | awk '{print $1}')
		sed -i "${line_number}s/=.*/= ${nx};/" ${input_file}
		echo sed -i "${line_number}s/=.*/= ${nx};/" ${input_file}
	else
		echo "${input_file} does not exist"
	fi
}


function set_ny_in_input_file(){
	local ny=${1}
	local input_file=${2}
	if [ -f ${input_file} ]; then
		local line_number=$(nl -b a ${input_file} | grep "[^a-z]ny .*=" | awk '{print $1}')
		sed -i "${line_number}s/=.*/= ${ny};/" ${input_file}
		echo sed -i "${line_number}s/=.*/= ${ny};/" ${input_file}
	else
		echo "${input_file} does not exist"
	fi
}


function set_Ux_in_input_file(){
	local Ux=${1}
	local input_file=${2}
	if [ -f ${input_file} ]; then
		local line_number=$(nl -b a ${input_file} | sed -e 's/ //g' | grep 'u=' | awk '{print $1}')
		sed -i "${line_number}s/=.*/= (${Ux},0,0);/" ${input_file}
		echo sed -i "${line_number}s/=.*/= (${Ux},0,0);/" ${input_file}
	else
		echo "${input_file} does not exist"
	fi
}


function submit_petsc_simulation(){
	local t=$(echo ${times} | awk -F':' '{print $1}' | xargs printf "%04d")
	for i in ${!nx_array[@]}
	do
		local nx=${nx_array[$i]}
		local ny=${ny_array[$i]}
		local corr=${volume_corr[$i]}
		local Ux=$(echo "scale=8; (0.01 * (${nx}/240))" | bc)
		local new_dir=Nx${nx}Ny${ny}
		if [ ! -d ${new_dir} ]; then
			mkdir -p ${new_dir}
			cp -v Input.in Hemera.sh ${ref_dir}/phi_${t}.dat ${ref_dir}/initialBubbleVolume.dat ${ref_dir}/bubbleNumber.dat ${new_dir}
			set_nx_in_input_file ${nx} ${new_dir}/Input.in
			set_ny_in_input_file ${ny} ${new_dir}/Input.in
			set_Ux_in_input_file ${Ux} ${new_dir}/Input.in
			cd ${new_dir}
			local alpha=$(echo "scale=8; (${corr})*(${nx}/240)^2" | bc)
			echo "alpha = ${alpha}"
			../petsc/CreateWall -i Input.in
			../petsc/InterpolateDM -f 'phi' -nb 96 -nxOld 240 -nyOld 150 -nxNew ${nx} -nyNew ${ny} -alpha ${alpha} -t ${t}
			mv -v phiNew_${t}.dat phi_${t}.dat
			mv -v initialBubbleVolumeNew.dat initialBubbleVolume.dat
			mv -v phiNewSum_${t}.vts phiSum_${t}.vts
			sed -i "s/--job-name=.*/--job-name=sim_$new_dir/" Hemera.sh
			echo "mpirun -n 16 ../petsc/Main -i Input.in -snes_type newtontr -snes_atol 1e-8 -snes_max_it 1000 -log_view -snes_monitor_short" >> Hemera.sh
			#sbatch Hemera.sh
			cd ..
		else
			echo "${new_dir} already exists"
		fi
		echo ''
	done
}


function get_bubble_contours()
{
	local tmin=$(echo ${times} | awk -F':' '{print $1}'); [ -z "${tmin}" ] && tmin="0"; tmin=$((tmin + 1))
	local tmax=$(echo ${times} | awk -F':' '{print $2}'); [ -z "${tmax}" ] && tmax=$(( ${tmin} + 1 ))
	local every=$(echo ${times} | awk -F':' '{print $3}'); [ -z ${every} ] && dt="1"
	for i in ${!nx_array[@]}
	do
		local nx=${nx_array[$i]}
		local ny=${ny_array[$i]}
		local new_dir=Nx${nx}Ny${ny}
		if [ -d ${new_dir} ]; then
			echo "python PostProcess.py -t ${tmin}:${tmax}:${every} -np ${np} > ${new_dir}/HemeraExport.sh"
			cp -v ${python_dir}/main.py ${python_dir}/util.py ${new_dir}
			python PostProcess.py -t ${tmin}:${tmax}:${every} -np ${np} > "${new_dir}/HemeraExport.sh"
			sed -i "s/main.py/main.py -nx ${nx} -ny ${ny}/g" ${new_dir}/HemeraExport.sh
			sed -i "s#./ExportBubbles#../petsc/ExportBubbles -wall#g" ${new_dir}/HemeraExport.sh
			sed -i '/ [E,e]xportBubbles/d' ${new_dir}/HemeraExport.sh #Space is important
			sed -i "s#--job-name=.*#--job-name=ctr_${new_dir}#" "${new_dir}/HemeraExport.sh"
		else
			echo "${new_dir} does not exist"
		fi
		echo ''
	done
}


function main(){
	pass_args $0 "$@"
	local my_petsc_dir=$(echo "${petsc_dir}" | sed 's#/# #g' | awk '{print $NF}')
	if [ ! -d "${my_petsc_dir}" ]; then
		cp -rv ${petsc_dir} . #Copy petsc stuff
		cd ${my_petsc_dir}
		make createWall interpolateDM exportBubbles main
		cd ..
	fi
	[ "${sim}" == "true" ] && submit_petsc_simulation
	[ "${ctr}" == "true" ] && get_bubble_contours
}


main "$@"
