#!/bin/bash


this_dir="$(dirname $(realpath "$0"))"
verbose="false"
petsc_dir="/home/gl5972/HOME/docs/2021-RA/SecondSubmission/petsc"
volume='200:401:25'
step0="false"
step1="false"
step2="false"
v_zero="250"


function print_usage()
{
	echo "Set initial simulations cases"
	echo "--help:          help"
	echo "--verbose:       verbose"
	echo "--petsc_dir:     directory where petsc code is stored"
	echo "--volume:        V0:V1:dV"
	echo "--step0:         start from square arranagement and random bubble size (V???_s0)"
	echo "--step1:         continue step 0 with constant volume (V???_s0 -> V???_s1)"
	echo "--step2-with-V0: continue step 1 by setting volume (V???_s1->V???_s2 and V???_s1->V???_s2r"
	echo "ex: (1) bash Initialize.sh --step0 --volume=250"
	echo "ex: (2) bash Initialize.sh --step1 --volume=250"
	echo "ex: (3) bash Initialize.sh --step2-with-V0=250 --volume='200:401:25'"
	exit
}


function pass_args()
{
	local shorts='hvi:p:a,b,c:'
	local longs='help,verbose,volume:,petsc_dir:,step0,step1,step2-with-V0:'
	local options=$(getopt --options ${shorts} --long ${longs} --name "$0" -- "$@")
	if [ $? != 0 ]; then echo "Failed parsing options." >&2 ; exit 1 ; fi
	eval set -- "${options}"
	while true; do
			case "$1" in
					-h | --help)          print_usage; shift ;;
					-v | --verbose)       verbose="true"; shift ;;
					-i | --volume)        volume=$2; shift 2;;
					-p | --petsc_dir)     petsc_dir=$2; shift 2;;
					-a | --step0)         step0="true" ; step1="false"; step2="false"; shift ;;
					-b | --step1)         step0="false"; step1="true" ; step2="false"; shift ;;
					-c | --step2-with-V0) step0="false"; step1="false"; step2="true" ; v_zero=$2; shift 2;;
					--) shift ; break ;;
					* ) break ;;
			esac
	done
}


function main()
{
	pass_args $0 "$@"
	local V0=$(echo ${volume} | awk -v FS=":" '{print $1}')
	local V1=$(echo ${volume} | awk -v FS=":" '{print $2}'); [ -z ${V1} ] && V1="(( ${V0} + 1 ))"
	local dV=$(echo ${volume} | awk -v FS=":" '{print $3}'); [ -z ${dV} ] && dV="25"
	for (( V=${V0}; V<${V1}; V+=${dV} )); do
		if [ ${step0} = "true" ]; then
			set_step0 ${V}
		fi
		if [ ${step1} = "true" ]; then
			set_step1 ${V}
		fi
		if [ ${step2} = "true" ]; then
			set_step2_with_constant_volume ${V} ${v_zero}
			set_step2_with_random_volume ${V} ${v_zero}
		fi
	done
	for f in V???_s2r; do
		[ -f ${f}/Hemera.sh ] && sed -i 's/intel/defq/' ${f}/Hemera.sh
	done
}


function set_step0(){
	local V=${1}
	local new_dir="V${V}_s0"
	if [ ! -d ${new_dir} ]; then
		echo "Set ${new_dir}"
		[ ${verbose} = "true" ] && local v='-v'
		[ ${verbose} = "false" ] && local dev_null='> /dev/null'
		eval mkdir ${v} -p ${new_dir}
		eval cp ${v} ${this_dir}/Input.in ${this_dir}/Hemera.sh ${new_dir}
		eval cp ${v} ${petsc_dir}/*\.h ${petsc_dir}/*\.cpp ${petsc_dir}/Makefile ${new_dir}
		cd ${new_dir}
			set_volume_in_input_file ${V} Input.in
			eval make NP=1 createFoam createVZero ${dev_null}
			eval ./CreateFoam -i Input.in ${dev_null}
			eval ./CreateVZero -i Input.in -rnd ${dev_null}
			eval make clean ${dev_null}
		cd ${this_dir}
		set_volume_in_input_file -1 ${new_dir}/Input.in
		set_restart_in_input_file 0 ${new_dir}/Input.in
		echo 'make NP=1 clean createWall main' >> ${new_dir}/Hemera.sh
		echo './CreateWall -i Input.in' >> ${new_dir}/Hemera.sh
		echo 'make NP=16 solve' >> ${new_dir}/Hemera.sh
		sed -i "s#--job-name=.*#--job-name=${new_dir}#" ${new_dir}/Hemera.sh
	else
		echo "${new_dir} already exists. Nothing to be done."
	fi
}


function set_step1()
{
	local V=${1}
	local new_dir="V${V}_s1"
	local old_dir="V${V}_s0"
	if [ -d ${old_dir} ] && [ ! -d ${new_dir} ]; then
		echo "Set ${new_dir}"
		[ ${verbose} = "true" ] && local v="-v"
		[ ${verbose} = "false" ] && local dev_null='> /dev/null'
		local lastTime=$(get_last_time ${old_dir}); [ ${verbose} = "true" ] && echo "lastTime(${old_dir}) = ${lastTime}"
		eval mkdir ${v} -p ${new_dir}
		eval cp ${v} ${this_dir}/Input.in ${this_dir}/Hemera.sh ${new_dir}
		eval cp ${v} ${petsc_dir}/*\.h ${petsc_dir}/*\.cpp ${petsc_dir}/Makefile ${new_dir}
		eval cp ${v} ${old_dir}/phi_${lastTime}.dat* ${old_dir}/phiWall.dat* ${old_dir}/bubbleNumber.dat* ${old_dir}/initialBubbleVolume.dat* ${new_dir}
		cd ${new_dir}
			set_volume_in_input_file ${V} Input.in
			eval make NP=1 createVZero ${dev_null}
			eval ./CreateVZero -i Input.in ${dev_null}
			eval make clean ${dev_null}
		cd ${this_dir}
		set_volume_in_input_file -1 ${new_dir}/Input.in
		set_restart_in_input_file ${lastTime} ${new_dir}/Input.in
		echo 'make NP=16 main solve' >> ${new_dir}/Hemera.sh
		sed -i "s#--job-name=.*#--job-name=${new_dir}#" ${new_dir}/Hemera.sh
	else
		echo "Making ${new_dir} is not feasible. Nothing to be done."
	fi
}


function set_step2_with_constant_volume()
{
	local V=${1}
	local new_dir="V${V}_s2"
	local fromV=${2}
	local old_dir="V${fromV}_s1"
	if [ -d ${old_dir} ] && [ ! -d ${new_dir} ]; then
		echo "Set ${new_dir}"
		[ ${verbose} = "true" ] && local v='-v'
		[ ${verbose} = "false" ] && local dev_null='> /dev/null'
		local lastTime=$(get_last_time ${old_dir}); [ ${verbose} = "true" ] && echo "lastTime(${old_dir}) = ${lastTime}"
		eval mkdir ${v} -p ${new_dir}
		eval cp ${v} ${this_dir}/Input.in ${this_dir}/Hemera.sh ${new_dir}
		eval cp ${v} ${petsc_dir}/*\.h ${petsc_dir}/*\.cpp ${petsc_dir}/Makefile ${new_dir}
		eval cp ${v} ${old_dir}/phi_${lastTime}.dat* ${old_dir}/phiWall.dat* ${old_dir}/bubbleNumber.dat* ${new_dir}
		cd ${new_dir}
			set_volume_in_input_file ${V} Input.in
			eval make NP=1 createVZero ${dev_null}
			eval ./CreateVZero -i Input.in ${dev_null}
			eval make clean ${dev_null}
		cd ${this_dir}
		set_volume_in_input_file -1 ${new_dir}/Input.in
		set_restart_in_input_file ${lastTime} ${new_dir}/Input.in
		echo 'make NP=16 main solve' >> ${new_dir}/Hemera.sh
		sed -i "s#--job-name=.*#--job-name=${new_dir}#" ${new_dir}/Hemera.sh
	else
		echo "Making ${new_dir} is not feasible. Nothing to be done."
	fi
}


function set_step2_with_random_volume()
{
	local V=${1}
	local new_dir="V${V}_s2r"
	local fromV=${2}
	local old_dir="V${fromV}_s1"
	if [ -d ${old_dir} ] && [ ! -d ${new_dir} ]; then
		echo "Set ${new_dir}"
		[ ${verbose} = "true" ] && local v='-v'
		[ ${verbose} = "false" ] && local dev_null='> /dev/null'
		local lastTime=$(get_last_time ${old_dir}); [ ${verbose} = "true" ] && echo "lastTime(${old_dir}) = ${lastTime}"
		eval mkdir ${v} -p ${new_dir}
		eval cp ${v} ${this_dir}/Input.in ${this_dir}/Hemera.sh ${new_dir}
		eval cp ${v} ${petsc_dir}/*\.h ${petsc_dir}/*\.cpp ${petsc_dir}/Makefile ${new_dir}
		eval cp ${v} ${old_dir}/phi_${lastTime}.dat* ${old_dir}/phiWall.dat* ${old_dir}/bubbleNumber.dat* ${new_dir}
		cd ${new_dir}
			set_volume_in_input_file ${V} Input.in
			eval make NP=1 create_vzero ${dev_null}
			eval ./CreateVZero -i Input.in -rnd ${dev_null}
			eval make clean ${dev_null}
		cd ${this_dir}
		set_volume_in_input_file -1 ${new_dir}/Input.in
		set_restart_in_input_file ${lastTime} ${new_dir}/Input.in
		echo 'make NP=16 main solve' >> ${new_dir}/Hemera.sh
		sed -i "s#--job-name=.*#--job-name=${new_dir}#" ${new_dir}/Hemera.sh
	else
		echo "Making ${new_dir} is not feasible. Nothing to be done."
	fi
}


function restart_case()
{
	if [ -d ${1} ]; then
		local lastTime=$(get_last_time ${1})
		set_restart_eq_in_input_file ${lastTime} ${1}/Input.in
	else
		echo "${1} not found. Restart not feasible."
	fi
}


function set_volume_in_input_file()
{
	local V=${1}
	local lineNumber=$(nl -b a $2 | grep "bubbleVolume.*=" | awk '{print $1}')
	sed -i "${lineNumber}s/=.*/= ${V};/" $2
	[ ${verbose} = "true" ] && echo volume changed to ${V} with sed -i "${lineNumber}s/=.*/= $V;/" $2
}


function set_restart_in_input_file(){
	local time=$(echo ${1} | sed 's/^0*//'); [ -z ${time} ] && time='0'
	local lineNumber=$(nl -b a $2 | grep "restart" | head -n 1 | awk '{print $1}')
	sed -i "${lineNumber}s/=.*/= ${time};/" $2
	[ ${verbose} = "true" ] && echo restart changed to ${time} with sed -i "${lineNumber}s/=.*/= ${time};/" $2
}


function set_n_in_input_file(){
	local nit=$(echo ${1} | sed 's/^0*//')
	local lineNumber=$(nl -b a $2 | grep "n" | head -n 1 | awk '{print $1}')
	sed -i "${lineNumber}s/=.*/= ${nit};/" $2
	[ ${verbose} = "true" ] && echo n changed to ${nit} with sed -i "${lineNumber}s/=.*/= ${nit};/" $2
}


get_last_time(){
	local myDir=${1}
	if [ -d ${myDir} ] && [ ! -z "$(find ${myDir} -maxdepth 1 -name phi_????.dat -type f)" ]; then
		local lastTime=$(ls -1 ${myDir}/phi_????.dat | tail -n 1 | awk -F'/' '{print $NF}' | sed -e 's/[^0-9]*//g')
		echo ${lastTime}
	else
		echo "none"
	fi
}


main "$@"
