#!/bin/bash


verbose="false"
volume='200:401:25'
raw_dir="/bigdata/capture/2020_JFM/JFM1/Unsteady"
petsc_dir="/home/gl5972/HOME/docs/2021-RA/SecondSubmission/petsc"
python_dir="/home/gl5972/HOME/docs/2021-RA/SecondSubmission/figures"
np="1"
times=""


function print_usage()
{
	echo "Convert petsc fields (phi_????.dat) into bubble contours for further processing"
	echo "--help:       help"
	echo "--verbose:    verbose"
	echo "--volume:     V0:V1:dV"
	echo "--raw_dir:    directory where raw data are stored"
	echo "--petsc_dir:  directory where petsc code is stored"
	echo "--python_dir: directory where python code is stored"
	echo "--np:         split the job into np processors"
	echo "--times:      from:to:every"
	exit
}


function pass_args()
{
	local shorts='hvi:r:p:y:n:t:'
	local longs='help,verbose,volume:,raw_dir:,petsc_dir:,python_dir:,np:,times:'
	local options=$(getopt --options ${shorts} --long ${longs} --name "$0" -- "$@")
	if [ $? != 0 ]; then echo "Failed parsing options." >&2 ; exit 1 ; fi
	eval set -- "${options}"
	while true; do
		case "$1" in
			-h | --help)       print_usage;    shift  ;;
			-v | --verbose)    verbose="true"; shift  ;;
			-i | --volume)     volume=$2;      shift 2;;
			-r | --raw_dir)    raw_dir=$2;     shift 2;;
			-p | --petsc_dir)  petsc_dir=$2;   shift 2;;
			-y | --python_dir) python_dir=$2;  shift 2;;
			-n | --np)         np=$2;          shift 2;;
			-t | --times)      times=$2;       shift 2;;
			--) shift ; break ;;
			* ) break ;;
		esac
	done
}


function main(){
	pass_args $0 "$@"
	local V0=$(echo ${volume} | awk -v FS=":" '{print $1}')
	local V1=$(echo ${volume} | awk -v FS=":" '{print $2}'); [ -z ${V1} ] && V1=$(( ${V0} + 1 ))
	local dV=$(echo ${volume} | awk -v FS=":" '{print $3}'); [ -z ${dV} ] && dV="25"
	for (( V=${V0}; V<${V1}; V+=${dV} )); do
		copy_data ${V}
		export_bubbles ${V}
	done
}


function copy_data()
{
	local V=${1}
	for vDir in "V${V}" "V${V}r"; do
		local old_dir=${raw_dir}/${vDir}
		local new_dir=${vDir}
#		if [ -d ${old_dir} ] && [ ! -d ${new_dir} ]; then
		if [ -d ${old_dir} ]; then
			local v=""; [ "${verbose}" == "true" ] && v="-v"
			eval mkdir ${v} -p ${new_dir}
			eval cp ${v} ${petsc_dir}/*\.cpp ${petsc_dir}/*\.h ${petsc_dir}/Makefile ${new_dir}
			eval cp ${v} ${python_dir}/util.py ${python_dir}/main.py ${new_dir}
			[ -f ${old_dir}/phiWall.dat ] && eval cp ${v} ${old_dir}/phiWall.dat ${new_dir}
			[ -f ${old_dir}/phiWall.dat.info ] && eval cp ${v} -sf ${old_dir}/phiWall.dat.info ${new_dir}
			[ -f ${old_dir}/Input.in ] && eval cp ${v} ${old_dir}/Input.in ${new_dir}
			[ -f ${old_dir}/bubbleNumber.dat ] && eval cp ${v} ${old_dir}/bubbleNumber.dat ${new_dir}
			[ -f ${old_dir}/bubbleNumber.dat.info ] && eval cp ${v} ${old_dir}/bubbleNumber.dat.info ${new_dir}
			[ -f ${old_dir}/initialBubbleVolume.dat ] && eval cp ${v} ${old_dir}/initialBubbleVolume.dat ${new_dir}
			[ -f ${old_dir}/initialBubbleVolume.dat.info ] && eval cp ${v} ${old_dir}/initialBubbleVolume.dat.info ${new_dir}
			for old_phi in ${old_dir}/phi_????.dat*
			do
				local new_phi=${new_dir}/$(echo ${old_phi} | awk -F'/' '{print $NF}')
				eval ln ${v} -sf ${old_phi} ${new_phi}
			done
		else
			echo "Making ${new_dir} is not feasible. Nothing to be done."
		fi
	done
}


function export_bubbles()
{
	local V=${1}
	for vDir in "V${V}" "V${V}r"; do
		local new_dir=${vDir}
#		if [ ! -f ${new_dir}/HemeraExport.sh ] && [ ! -f ${new_dir}/HemeraClean.sh ]; then
		if [ -d ${new_dir} ]; then
			if [ -z ${times} ]; then
				local tmin=$(ls -1 ${new_dir}/phi_????.dat | head -n 1 | awk -F'/' '{print $NF}' | sed -e 's/[^0-9]*//g' -e 's/^0*//'); [ -z "${tmin}" ] && tmin="0"; tmin=$((tmin + 1))
				local tmax=$(ls -1 ${new_dir}/phi_????.dat | tail -n 1 | awk -F'/' '{print $NF}' | sed -e 's/[^0-9]*//g' -e 's/^0*//'); tmax=$((tmax + 1))
				local every="1"
			else
				local tmin=$(echo ${times} | awk -v FS=":" '{print $1}')
				local tmax=$(echo ${times} | awk -v FS=":" '{print $2}'); [ -z ${tmax} ] && tmax=$(( ${tmin} + 1 ))
				local every=$(echo ${times} | awk -v FS=":" '{print $3}'); [ -z ${every} ] && every="1"
			fi
			if [ "${verbose}" == "true" ]; then
				echo "${HOME}/anaconda3/bin/python PostProcess.py -t ${tmin}:${tmax}:${every} -np ${np} > ${new_dir}/HemeraExport.sh"
				echo "${HOME}/anaconda3/bin/python PostProcess.py -clean y  > ${new_dir}/HemeraClean.sh"
			fi
			${HOME}/anaconda3/bin/python PostProcess.py -t ${tmin}:${tmax}:${every} -np ${np} > "${new_dir}/HemeraExport.sh"
			${HOME}/anaconda3/bin/python PostProcess.py -clean y > "${new_dir}/HemeraClean.sh"
			sed -i "s#--job-name=.*#--job-name=${vDir}#" "${new_dir}/HemeraExport.sh"
			sed -i "s#--job-name=.*#--job-name=${vDir}_clean#" "${new_dir}/HemeraClean.sh"
		else
			echo "Making hemera files in ${new_dir} is not feasible. Nothing to be done."
		fi
	done
}


main "$@"


#for f in V*; do echo $f  $(ls $f/bb_* | wc -w); done
#for f in V200/*; do [ ! -f V300/$(echo $f | awk -F'/' '{print $NF}') ] && echo $f | awk -F'/' '{print $NF}'; done
