#!/bin/bash


this_dir="$(dirname $(realpath "$0"))"
petsc_dir="/home/gl5972/HOME/docs/2021-RA/SecondSubmission/petsc"
init_dir="/bigdata/capture/2020_JFM/JFM1/Initialization"
volume="200:401:25"
verbose="false"
restart="false"


function print_usage()
{
	echo "Set unsteady simulations cases"
	echo "--help:       help"
	echo "--verbose:    verbose"
	echo "--volume:     V0:V1:dV"
	echo "--petsc_dir:  directory where petsc code is stored"
	echo "--init_dir:   directory where initialization data are stored"
	echo "--restart:    restart from last time step"
	echo "ex: bash Unsteady.sh --petsc_dir=/home/gl5972/HOME/docs/2021-RA/SecondSubmission/petsc --volume='200:401:25' --verbose"
	exit
}


function pass_args()
{
	local shorts='hvi:p:n:r'
	local longs='help,verbose,volume:,petsc_dir:,init_dir:,restart'
	local options=$(getopt --options ${shorts} --long ${longs} --name "$0" -- "$@")
	if [ $? != 0 ]; then echo "Failed parsing options." >&2 ; exit 1 ; fi
	eval set -- "${options}"
	while true; do
			case "$1" in
				-h | --help)       print_usage;    shift  ;;
				-v | --verbose)    verbose="true"; shift  ;;
				-i | --volume)     volume=$2;      shift 2;;
				-p | --petsc_dir)  petsc_dir=$2;   shift 2;;
				-n | --init_dir)   init_dir=$2;    shift 2;;
				-r | --restart)    restart="true"; shift  ;;
				--) shift ; break ;;
				* ) break ;;
			esac
	done
}


main(){
	pass_args $0 "$@"
	V0=$(echo ${volume} | awk -v FS=":" '{print $1}')
	V1=$(echo ${volume} | awk -v FS=":" '{print $2}'); [ -z "${V1}" ] && V1="(( $V0 + 1 ))"
	dV=$(echo ${volume} | awk -v FS=":" '{print $3}'); [ -z "${dV}" ] && dV="25"
	for (( V=${V0}; V<${V1}; V+=${dV} )); do
		if [ ${restart} = "true" ]; then
			set_restart_in_input_file $(get_last_time V${V}) V${V}/Input.in
			set_restart_in_input_file $(get_last_time V${V}r) V${V}r/Input.in
		else
			set_case_with_constant_volume ${V};
			set_case_with_random_volume ${V};
		fi
	done
	for V in V???r; do sed -i 's/intel/defq/' $V/Hemera.sh; done
}


function set_case_with_constant_volume(){
	local V=${1}
	local old_dir=${init_dir}/V${V}_s2
	local new_dir=${this_dir}/V${V}
	if [ -d ${old_dir} ] && [ ! -d ${new_dir} ]; then
		[ ${verbose} = "true" ] && local v="-v"
		local lastTime=$(get_last_time ${old_dir}); [ ${verbose} = "true" ] && echo "lastTime(${old_dir}) = ${lastTime}"
		eval mkdir ${v} -p ${new_dir}
		eval cp ${v} ${this_dir}/Input.in ${this_dir}/Hemera.sh ${new_dir}
		eval cp ${v} ${petsc_dir}/*\.h ${petsc_dir}/*\.cpp ${petsc_dir}/Makefile ${new_dir}
		eval cp ${v} ${old_dir}/phi_${lastTime}.dat ${new_dir}/phi_0000.dat
		eval cp ${v} ${old_dir}/phi_${lastTime}.dat.info ${new_dir}/phi_0000.dat.info
		eval cp ${v} ${old_dir}/phiWall.dat* ${old_dir}/bubbleNumber.dat* ${old_dir}/initialBubbleVolume.dat* ${new_dir}
		set_restart_in_input_file 0 ${new_dir}/Input.in
		echo 'make NP=16 main solve' >> ${new_dir}/Hemera.sh
		sed -i "s#--job-name=.*#--job-name=V${V}#" ${new_dir}/Hemera.sh
	else
		echo "Making ${new_dir} is not feasible. Nothing to be done."
	fi
}


function set_case_with_random_volume(){
	local V=${1}
	local old_dir=${init_dir}/V${V}_s2r
	local new_dir=${this_dir}/V${V}r
	if [ -d ${old_dir} ] && [ ! -d ${new_dir} ]; then
		[ ${verbose} = "true" ] && local v="-v"
		local lastTime=$(get_last_time ${old_dir}); [ ${verbose} = "true" ] && echo "lastTime(${old_dir}) = ${lastTime}"
		eval mkdir ${v} -p ${new_dir}
		eval cp ${v} ${this_dir}/Input.in ${this_dir}/Hemera.sh ${new_dir}
		eval cp ${v} ${petsc_dir}/*\.h ${petsc_dir}/*\.cpp ${petsc_dir}/Makefile ${new_dir}
		eval cp ${v} ${old_dir}/phi_${lastTime}.dat ${new_dir}/phi_0000.dat
		eval cp ${v} ${old_dir}/phi_${lastTime}.dat.info ${new_dir}/phi_0000.dat.info
		eval cp ${v} ${old_dir}/phiWall.dat* ${old_dir}/bubbleNumber.dat* ${old_dir}/initialBubbleVolume.dat* ${new_dir}
		set_restart_in_input_file 0 ${new_dir}/Input.in
		echo 'make NP=16 main solve' >> ${new_dir}/Hemera.sh
		sed -i "s#--job-name=.*#--job-name=V${V}r#" ${new_dir}/Hemera.sh
	else
		echo "Making ${new_dir} is not feasible. Nothing to be done."
	fi
}


function set_volume_in_input_file()
{
	local V=${1}
	local inputFile=${2}
	if [ -f ${inputFile} ]; then
		local lineNumber=$(nl -b a ${inputFile} | grep "bubbleVolume.*=" | awk '{print $1}')
		sed -i "${lineNumber}s/=.*/= $V;/" ${inputFile}
		[ ${verbose} = "true" ] && echo sed -i "${lineNumber}s/=.*/= $V;/" ${inputFile}
	else
		echo "${inputFile} does not exist"
	fi
}


function set_restart_in_input_file(){
	local t=${1}
	local inputFile=${2}
	if [ -f ${inputFile} ]; then
		local time=$(echo ${t} | sed 's/^0*//'); [ -z ${time} ] && time='0'
		local lineNumber=$(nl -b a ${inputFile} | grep "[^a-z]restart .*=" | sed '2q;d' | awk '{print $1}')
		sed -i "${lineNumber}s/=.*/= ${time};/" ${inputFile}
		[ ${verbose} = "true" ] && echo sed -i "${lineNumber}s/=.*/= ${time};/" ${inputFile}
	else
		echo "${inputFile} does not exist"
	fi
}


function set_n_in_input_file(){
	local n=${1}
	local inputFile=${2}
	if [ -f ${inputFile} ]; then
		local nit=$(echo $1 | sed 's/^0*//')
		local lineNumber=$(nl -b a ${inputFile} | grep "[^a-z]n .*=" | sed '2q;d' | awk '{print $1}')
		sed -i "${lineNumber}s/=.*/= ${nit};/" ${inputFile}
		[ ${verbose} = "true" ] && echo sed -i "${lineNumber}s/=.*/= ${nit};/" ${inputFile}
	else
		echo "${inputFile} does not exist"
	fi
}


get_last_time(){
	local myDir=${1}
	if [ -d ${myDir} ] && [ ! -z "$(find ${myDir} -maxdepth 1 -name phi_????.dat -type f)" ] ; then
		local lastTime=$(ls -1 ${myDir}/phi_????.dat | tail -n 1 | awk -F'/' '{print $NF}' | sed -e 's/[^0-9]*//g')
		echo ${lastTime}
	else
		echo "none"
	fi
}


main "$@"
